@@ -15,8 +15,6 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-
-
 import datetime
 import json
 import logging
@@ -26,11 +24,13 @@ from time import sleep
 
 from sqlalchemy import and_, or_, tuple_
 
+from airflow.compat.functools import cached_property
 from airflow.exceptions import AirflowException, AirflowTaskTimeout
-from airflow.models import BaseOperator, SensorInstance, SkipMixin, TaskInstance
+from airflow.models import BaseOperator, DagRun, SensorInstance, SkipMixin, TaskInstance
 from airflow.settings import LOGGING_CLASS_PATH
 from airflow.stats import Stats
 from airflow.utils import helpers, timezone
+from airflow.utils.context import Context
 from airflow.utils.email import send_email
 from airflow.utils.log.logging_mixin import set_context
 from airflow.utils.module_loading import import_string
@@ -43,7 +43,7 @@ config = import_string(LOGGING_CLASS_PATH)
 handler_config = config['handlers']['task']
 try:
     formatter_config = config['formatters'][handler_config['formatter']]
-except Exception as err:  # pylint: disable=broad-except
+except Exception as err:
     formatter_config = None
     print(err)
 dictConfigurator = DictConfigurator(config)
@@ -77,16 +77,12 @@ class SensorWork:
 
         self.poke_context = json.loads(si.poke_context) if si.poke_context else {}
         self.execution_context = json.loads(si.execution_context) if si.execution_context else {}
-        try:
-            self.log = self._get_sensor_logger(si)
-        except Exception as e:  # pylint: disable=broad-except
-            self.log = None
-            print(e)
         self.hashcode = si.hashcode
         self.start_date = si.start_date
         self.operator = si.operator
         self.op_classpath = si.op_classpath
         self.encoded_poke_context = si.poke_context
+        self.si = si
 
     def __eq__(self, other):
         if not isinstance(other, SensorWork):
@@ -105,7 +101,7 @@ class SensorWork:
         Create task log handler for a sensor work.
         :return: log handler
         """
-        from airflow.utils.log.secrets_masker import _secrets_masker  # noqa
+        from airflow.utils.log.secrets_masker import _secrets_masker
 
         handler_config_copy = {k: handler_config[k] for k in handler_config}
         del handler_config_copy['filters']
@@ -119,15 +115,17 @@ class SensorWork:
         handler.addFilter(_secrets_masker())
         return handler
 
-    def _get_sensor_logger(self, si):
+    @cached_property
+    def log(self):
         """Return logger for a sensor instance object."""
         # The created log_id is used inside of smart sensor as the key to fetch
         # the corresponding in memory log handler.
+        si = self.si
         si.raw = False  # Otherwise set_context will fail
         log_id = "-".join(
             [si.dag_id, si.task_id, si.execution_date.strftime("%Y_%m_%dT%H_%M_%S_%f"), str(si.try_number)]
         )
-        logger = logging.getLogger('airflow.task' + '.' + log_id)
+        logger = logging.getLogger(f'airflow.task.{log_id}')
 
         if len(logger.handlers) == 0:
             handler = self.create_new_task_handler()
@@ -147,7 +145,7 @@ class SensorWork:
         for handler in self.log.handlers:
             try:
                 handler.close()
-            except Exception as e:  # pylint: disable=broad-except
+            except Exception as e:
                 print(e)
 
     @property
@@ -252,10 +250,7 @@ class SensorExceptionInfo:
             self._infra_failure_timeout = timezone.utcnow() + self._infra_failure_retry_window
 
     def should_fail_current_run(self):
-        """
-        :return: Should the sensor fail
-        :type: boolean
-        """
+        """:return: Should the sensor fail"""
         return not self.is_infra_failure or timezone.utcnow() > self._infra_failure_timeout
 
     @property
@@ -265,18 +260,11 @@ class SensorExceptionInfo:
 
     @property
     def is_infra_failure(self):
-        """
-
-        :return: If the exception is an infra failure
-        :type: boolean
-        """
+        """:return: If the exception is an infra failure"""
         return self._is_infra_failure
 
     def is_expired(self):
-        """
-        :return: If current exception need to be kept.
-        :type: boolean
-        """
+        """:return: If current exception need to be kept."""
         if not self._is_infra_failure:
             return True
         return timezone.utcnow() > self._infra_failure_timeout + datetime.timedelta(minutes=30)
@@ -293,19 +281,13 @@ class SmartSensorOperator(BaseOperator, SkipMixin):
     all sensor task state in task_instance table
 
     :param soft_fail: Set to true to mark the task as SKIPPED on failure
-    :type soft_fail: bool
     :param poke_interval: Time in seconds that the job should wait in
         between each tries.
-    :type poke_interval: int
     :param smart_sensor_timeout: Time, in seconds before the internal sensor
         job times out if poke_timeout is not defined.
-    :type smart_sensor_timeout: float
     :param shard_min: shard code lower bound (inclusive)
-    :type shard_min: int
     :param shard_max: shard code upper bound (exclusive)
-    :type shard_max: int
     :param poke_timeout: Time, in seconds before the task times out and fails.
-    :type poke_timeout: float
     """
 
     ui_color = '#e6f1f2'
@@ -371,7 +353,7 @@ class SmartSensorOperator(BaseOperator, SkipMixin):
         for ti in tis:
             try:
                 sensor_works.append(SensorWork(ti))
-            except Exception:  # pylint: disable=broad-except
+            except Exception:
                 self.log.exception("Exception at creating sensor work for ti %s", ti.key)
 
         self.log.info("%d tasks detected.", len(sensor_works))
@@ -390,24 +372,35 @@ class SmartSensorOperator(BaseOperator, SkipMixin):
         :param sensor_works: Smart sensor internal object for a sensor task.
         :param session: The sqlalchemy session.
         """
+        DR = DagRun
         TI = TaskInstance
-        ti_keys = [(x.dag_id, x.task_id, x.execution_date) for x in sensor_works]
 
-        def update_ti_hostname_with_count(count, ti_keys):
+        def update_ti_hostname_with_count(count, sensor_works):
             # Using or_ instead of in_ here to prevent from full table scan.
-            tis = (
-                session.query(TI)
-                .filter(or_(tuple_(TI.dag_id, TI.task_id, TI.execution_date) == ti_key for ti_key in ti_keys))
-                .all()
-            )
+            if session.bind.dialect.name == 'mssql':
+                ti_filter = or_(
+                    and_(
+                        TI.dag_id == ti_key.dag_id,
+                        TI.task_id == ti_key.task_id,
+                        DR.execution_date == ti_key.execution_date,
+                    )
+                    for ti_key in sensor_works
+                )
+            else:
+                ti_keys = [(x.dag_id, x.task_id, x.execution_date) for x in sensor_works]
+                ti_filter = or_(
+                    tuple_(TI.dag_id, TI.task_id, DR.execution_date) == ti_key for ti_key in ti_keys
+                )
 
-            for ti in tis:
+            for ti in session.query(TI).join(TI.dag_run).filter(ti_filter):
                 ti.hostname = self.hostname
             session.commit()
 
-            return count + len(ti_keys)
+            return count + len(sensor_works)
 
-        count = helpers.reduce_in_chunks(update_ti_hostname_with_count, ti_keys, 0, self.max_tis_per_query)
+        count = helpers.reduce_in_chunks(
+            update_ti_hostname_with_count, sensor_works, 0, self.max_tis_per_query
+        )
         if count:
             self.log.info("Updated hostname on %s tis.", count)
 
@@ -435,6 +428,7 @@ class SmartSensorOperator(BaseOperator, SkipMixin):
         TI = TaskInstance
 
         count_marked = 0
+        query_result = []
         try:
             query_result = (
                 session.query(TI, SI)
@@ -468,7 +462,7 @@ class SmartSensorOperator(BaseOperator, SkipMixin):
 
             session.commit()
 
-        except Exception:  # pylint: disable=broad-except
+        except Exception:
             self.log.warning(
                 "Exception _mark_multi_state in smart sensor for hashcode %s",
                 str(poke_hash),  # cast to str in advance for highlighting
@@ -484,9 +478,7 @@ class SmartSensorOperator(BaseOperator, SkipMixin):
         logically experienced all retries and the try_number should be set to max_tries.
 
         :param sensor_work: The sensor_work with exception.
-        :type sensor_work: SensorWork
         :param error: The error message for this sensor_work.
-        :type error: str.
         :param session: The sqlalchemy session.
         """
 
@@ -496,7 +488,7 @@ class SmartSensorOperator(BaseOperator, SkipMixin):
                 email = sensor_work.execution_context.get('email')
 
                 send_email(email, subject, html_content)
-            except Exception:  # pylint: disable=broad-except
+            except Exception:
                 sensor_work.log.warning("Exception alerting email.", exc_info=True)
 
         def handle_failure(sensor_work, ti):
@@ -656,7 +648,7 @@ class SmartSensorOperator(BaseOperator, SkipMixin):
                     self._check_and_handle_ti_timeout(sensor_work)
 
                 self.cached_sensor_exceptions.pop(cache_key, None)
-        except Exception as e:  # pylint: disable=broad-except
+        except Exception as e:
             # The retry_infra_failure decorator inside hive_hooks will raise exception with
             # is_infra_failure == True. Long poking timeout here is also considered an infra
             # failure. Other exceptions should fail.
@@ -727,10 +719,10 @@ class SmartSensorOperator(BaseOperator, SkipMixin):
             Stats.gauge("smart_sensor_operator.poked_exception", count_poke_exception)
             Stats.gauge("smart_sensor_operator.exception_failures", count_exception_failures)
             Stats.gauge("smart_sensor_operator.infra_failures", count_infra_failure)
-        except Exception:  # pylint: disable=broad-except
+        except Exception:
             self.log.exception("Exception at getting loop stats %s")
 
-    def execute(self, context):
+    def execute(self, context: Context):
         started_at = timezone.utcnow()
 
         self.hostname = get_hostname()
@@ -746,23 +738,20 @@ class SmartSensorOperator(BaseOperator, SkipMixin):
             for sensor_work in self.sensor_works:
                 self._execute_sensor_work(sensor_work)
 
-            duration = (timezone.utcnow() - poke_start_time).total_seconds()
+            duration = timezone.utcnow() - poke_start_time
+            duration_seconds = duration.total_seconds()
 
-            self.log.info("Taking %s to execute %s tasks.", duration, len(self.sensor_works))
+            self.log.info("Taking %s seconds to execute %s tasks.", duration_seconds, len(self.sensor_works))
 
             Stats.timing("smart_sensor_operator.loop_duration", duration)
             Stats.gauge("smart_sensor_operator.executed_tasks", len(self.sensor_works))
             self._emit_loop_stats()
 
-            if duration < self.poke_interval:
-                sleep(self.poke_interval - duration)
+            if duration_seconds < self.poke_interval:
+                sleep(self.poke_interval - duration_seconds)
             if (timezone.utcnow() - started_at).total_seconds() > self.timeout:
                 self.log.info("Time is out for smart sensor.")
                 return
 
     def on_kill(self):
         pass
-
-
-if __name__ == '__main__':
-    SmartSensorOperator(task_id='test').execute({})
